#<profile4xi profile = "20" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "pipeclient4xi.py"> 
#<abstract4xi abstract = "provides the function PipeClient4Xi"/> 
#<refer4xi reference = "connection.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
#<note4xi note = "function name starts with capital"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'PipeClient4Xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import errno
import io
import itertools
import os
import pickle
import select
import socket
import struct
import sys
import time
import tempfile
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import connection
try:                             # Python ones
    from multiprocessing.connection import PipeClient
except:
    print("pipeclient4xi.py *** failed in from multiprocessing.connection import PipeClient ***")
try:                             # Python ones
    from multiprocessing.connection import PipeConnection
except:
    print("pipeclient4xi.py *** failed in from multiprocessing.connection import PipeConnection ***")    
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from constant_system4xi import get_windows_system4xi
from constant_family4xi import get_af_inet4xi
from constant_family4xi import get_af_unix4xi
from constant_family4xi import get_af_pipe4xi
from loop4xi import limit2loop4xi
from print4xi import print4xi
from sleep4xi import omega2sleep4xi
from time4xi import time4xi
from timeout4xi import check_connectiontimeout4xi
from timeout4xi import omega2timeout4xi
#"
#/>

 

# -------------------------------------------------------------------------------------------------
# PipeClient4Xi functions for different platforms                              
# -------------------------------------------------------------------------------------------------
if sys.platform == get_windows_system4xi():

    # -------------------------------------------------------------------------------------------------
    #<function4xi function = "PipeClient4Xi" usage = "
    #  client4xi.py:
    #     connection2xi = PipeClient4Xi(exchange)
    #"> 
    #<note4xi note = "even with Windows Vista64-bit this is the version used!"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # -------------------------------------------------------------------------------------------------
    def PipeClient4Xi(wave2pipeclient):
        '''
        Returns a connection object connected to the pipe given by wave2pipeclient.
        '''
        # -------------------------------------------------------------------------------------    
        # (Step 1/4)
        # -------------------------------------------------------------------------------------         

        class2print = "PipeClient4Xi"
        method2print = "PipeClient4Xi" 
        
        message2print = "{(Step 1/4a) started" + "; PipeClient4Xi-started: " + str(PipeClient4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        message2print = "{(Step 1/4b) passed" + "; wave2pipeclient-passed-to-PipeClient4Xi: " + str(wave2pipeclient) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # None *** JATKA to be solved ***          
        
        if wave2pipeclient is None:
            message2print = "{(Step 1/4c) hit" + "; wave2pipeclient-hit-by-PipeClient4Xi: " + " *** " + str(wave2pipeclient)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)

            
            
                            
        # -------------------------------------------------------------------------------------    
        # (Step 2/4)
        # -------------------------------------------------------------------------------------         
        message2print = "{looping" + "; WaitNamedPipe-looping-by-PipeClient4Xi: " + str(_winapi.WaitNamedPipe) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <built-in function WaitNamedPipe>       
        
        while 1:
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
            
            try:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                message2print = "{(Step 2/4a) passing" + "; wave2pipeclient-passing-to-winapi-WaitNamedPipe-by-PipeClient4Xi: " + str(wave2pipeclient) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
                    
                timeout2pipe = omega2sleep4xi(class2print, method2print) 
        
                message2print = "{(Step 2/4b) passing" + "; timeout2pipe-passing-to-winapi-WaitNamedPipe-by-PipeClient4Xi: " + str(timeout2pipe) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                        
                count2wait = 0
                ack2wait = False
                nack2wait = True

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                max2count = limit2loop4xi(class2print, method2print) 
                message2print = "{(Step 2/4c) got" + "; max2count-got-by-deliver-challenge4xi: " + str(max2count)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)        
                   
                while((count2wait < max2count) and (ack2wait is False)):
                
                    try:    
                                                 
                        _winapi.WaitNamedPipe(wave2pipeclient, timeout2pipe)
                    
                        ack2wait = True
                        nack2wait = False
                        
                    except:
                        message2print = "{(Step 2/4d) yielded" + "; nack2wait-yielded-by-PipeClient4Xi: " + str(nack2wait)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                                        
                        count2wait = count2wait + 1
                        
                        omega2sleep4xi(class2print, method2print)
                        
                         
                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                           
        
                message2print = "{(Step 2/4e) passing" + "; wave2pipeclient-passing-to-_winapi-CreateFile-by-PipeClient4Xi: " + str(wave2pipeclient) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
           
                message2print = "{(Step 2/4f) passing" + "; GENERIC-passing-to-_winapi-CreateFile-by-PipeClient4Xi: " + str(_winapi.GENERIC_READ | _winapi.GENERIC_WRITE) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
        
        
                message2print = "{(Step 2/4g) passing" + "; 0-passing-to-_winapi-CreateFile-by-PipeClient4Xi: " + str(0) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
        
                message2print = "{(Step 2/4h) passing" + "; _winapi-NULL-passing-to-_winapi-CreateFile-by-PipeClient4Xi: " + str(_winapi.NULL) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
        
                message2print = "{(Step 2/4i) passing" + "; _winapi-OPEN_EXISTING-passing-to-_winapi-CreateFile-by-PipeClient4Xi: " + str(_winapi.OPEN_EXISTING) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
        
                message2print = "{(Step 2/4j) passing" + "; 0-passing-to-_winapi-CreateFile-by-PipeClient4Xi: " + str(0) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
        
                message2print = "{(Step 2/4k) passing" + "; _winapi-NULL-passing-to-_winapi-CreateFile-by-PipeClient4Xi: " + str(_winapi.NULL) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
                                              
                

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                   
                count2wait = 0
                ack2handle = False
                nack2handle = True

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                   
                while((count2wait < max2count) or (ack2wait is False)):
                
                    try:    
                                                 
                        h2xi = _winapi.CreateFile(wave2pipeclient, _winapi.GENERIC_READ | _winapi.GENERIC_WRITE,
                                                      0, _winapi.NULL, _winapi.OPEN_EXISTING, 0, _winapi.NULL)

                        message2print = "{(Step 2/4l) got" + "; handle2xi-got-by-PipeClient4Xi: " + str(h2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                         
                        nack2handle = False
                        ack2handle = True
                        
                    except:
                        message2print = "{(Step 2/4m) yielded" + "; nack2handle-yielded-by-PipeClient4Xi: " + str(nack2handle)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                                        
                        count2wait = count2wait + 1
                        
                        omega2sleep4xi(class2print, method2print)
                         
                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                                                 
                                               
                    
            except WindowsError as e2xi:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                if e2xi.args[0] not in (_winapi.ERROR_SEM_TIMEOUT,
                                      _winapi.ERROR_PIPE_BUSY) or check_connectiontimeout4xi(timeout2pipe):
    
                    # -----------------------------------------------------------------------------    
                    #
                    # ----------------------------------------------------------------------------- 

                    message2print = "{(Step 2/4n) hit" + "; WindowsError-hit-by-PipeClient4Xi: " + " *** PipeClient4Xi: _winapi.ERROR_SEM_TIMEOUT _winapi.ERROR_PIPE_BUSY *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                                                   
                    raise
            else:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                break
        else:
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
    
            message2print = "{(Step 2/4o) raising" + "; while-not-1-raising-by-PipeClient4Xi: " + "** PipeClient4Xi: when not 1 in while **" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                              
            raise
             
        # -------------------------------------------------------------------------------------    
        # (Step 3/4)
        # -------------------------------------------------------------------------------------         
        message2print = "{(Step 3/4a) passing" + "; handle2xi-passing-to-_winapi-SetNamedPipeHandleState-by-PipeClient4Xi: " + str(h2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        message2print = "{(Step 3/4b) passing" + "; _winapi-PIPE-READMODE-MESSAGE-passing-to-_winapi-SetNamedPipeHandleState-by-PipeClient4Xi: " + str(_winapi.PIPE_READMODE_MESSAGE) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 3/4c) passing" + "; None-passing-to-_winapi-SetNamedPipeHandleState-by-PipeClient4Xi: " + str(None) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 3/4d) passing" + "; None-passing-to-_winapi-SetNamedPipeHandleState-by-PipeClient4Xi: " + str(None) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                   
        
    

        count2wait = 0
        ack2state = False
        nack2state = True
        # ---------------------------------------------------------------------------------    
        #
        # ---------------------------------------------------------------------------------     
           
        while((count2wait < max2count) or (ack2wait is False)):
        
            try:    
                                         
                _winapi.SetNamedPipeHandleState(h2xi, _winapi.PIPE_READMODE_MESSAGE, None, None)
                
                message2print = "{(Step 3/4e) performed" + "; _winapi-SetNamedPipeHandleState-performed-by-PipeClient4Xi: " + str(_winapi.SetNamedPipeHandleState) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)      
            
                nack2state = False
                ack2state = True
                
            except:
                message2print = "{(Step 2/4c) yielded" + "; nack2handle-yielded-by-PipeClient4Xi: " + str(nack2handle)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                          
                count2wait = count2wait + 1 
                
                omega2sleep4xi(class2print, method2print)
            
        # -------------------------------------------------------------------------------------    
        # (Step 4/4)
        # -------------------------------------------------------------------------------------         
        message2print = "{(Step 4/4a) passing" + "; handle2xi-passing-to-PipeConnection-by-PipeClient4Xi: " + str(h2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
         
        message2print = "{(Step 4/4b)returning" + "; PipeConnection-function-returning-by-PipeClient4Xi: " + str(PipeConnection) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
                    
        return PipeConnection(h2xi)
        
#</function4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_current_process4xi.txt;
#                                *** to be solved: why is wave2pipeclient None - see and test_current_process4xi.txtace-test-current-process4xi.xml!!! ***
#   sense-current-process4xi.xml
#   reason-current-process4xi.xml
#   ace-current-process4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.wave2pipeclientxi.com; www.yourwave2pipeclientxi.com; www.yourwave2pipeclientxi.org; www.wave2pipeclient2xi.com/; www.wave2pipeclient4xi.com/; www.wave2pipeclient8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of pipeclient4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>



